function nma_PCAandICA
%function nma_PCAandICA
%
% This is the main function to perform cancer detection
% accuracy analysis using the ICA algorithm

%short description: The matlab file requires that you have the following
%text files in the same folder as this file:
%
%      imputed_bladder.txt
%      imputed_liver.txt
%
%Next, you need to set a flag (see below) to indicate which of the above
%files to process. Only one file can be processed at one time.
%
%At the start of the file below, there is a CONFIGURATION section.
%one only need to edit this section before running this file.

%change history
% 041607 nabbasi added stats to check for sample classification
% 041907 nabbasi added logic to count how many samples go to correct bin
% 041907 nabbasi added support for more than one mode in doStats
% 042007 nabbasi modification of 3rd algorithm (combination) on how to
%                count accuracy, changes per Dr Lee input.
% 042007 nabbasi changed the order of check in the 3rd algorithm
% 060207 nabbasi Clean up code more and add more comments
%                before meeting with Dr Lee on 6/8/07
% 061507 nabbasi add ICA support, other rewrites. This code calls the
%                ICA code written by Michael Vodhanel
% 062907 nabbasi rewrite so that the same dataset used for PCA accuracy
%                analysis is also used for ICA

% Nasser Abbasi

clear all
close all
pause(1); %wait for windows to close, there seems to be bug in new matlab

%%%%%%%%% CONFIGURATION SECTION, EDIT ONLY THIS %%%%%%%%%%%%%%%%

MULTI_MODE_ADD=1;  %set this to 1 if when using more than one mode
                   %then add the effect of each mode to get total
                   %projection. If set to zero, then we do not add the
                   %projections, we just find the projection one each mode
                   %separatly
                   
REMOVE_MEAN_FROM_RAW_DATA = 1;  %do not change
USE_BLADDER_DATA          = 1;  %make this 1 to use LIVER data

PROJ_ON_TUMOR  = 1;  % do not change
PROJ_ON_NORMAL = 2;  % do not change

DO_CUT_OF_TEST = 0;  % change to 1 to do the excluded set test, i.e.
% changing the size of excluded set to see its
% effect on accuracy. By making this 1, the code
% will run longer, trying different working set size
% as % of total population. For each such working set
% it will run it NTEST times, each time selecting
% by random a working set of the current size.
% Make this ZERO for normal work. No need to do this
% unless interested in seeing how changing the
% working set affects accuracy

NTEST = 100;  %number of tests: number of projection matrices to generate
NMODE  = 6;  %This is the number of primary eignemodes to use.
nfig   = 1;  %This is figure counter. Leave at 1.

if DO_CUT_OF_TEST
    CUTOFF = 0.05:0.20:0.95;
else
    CUTOFF = .75:.75; % percentage of samples to remove before
    % using the rest for eigensample. Use 70% of
    % population to generate eigenmode.
end

if USE_BLADDER_DATA
    trueNormalIndex =  103:124 ; %bladder
    trueTumorIndex  = [1:6,8:102,125,126]; %bladder
else
    trueTumorIndex  = [4,9:112];  %liver
    trueNormalIndex = 116:191;  %liver
end

%%%%%%%%%%%%%%%%%%%% END CONFIGURATION %%%%%%%%%%%%%%%%%%%%%%%%%

if USE_BLADDER_DATA
    fprintf('***** BLADDER DATA ANALYSIS ********\n');
    xraw = load('imputed_bladder.txt');
else
    fprintf('***** LIVER DATA ANALYSIS ********\n');
    xraw = load('imputed_liver.txt');
end

if REMOVE_MEAN_FROM_RAW_DATA
    nrx=size(xraw,1);
    for i=1:nrx
        xraw(i,:)=xraw(i,:)-mean(xraw(i,:));
    end
end

nCutOff=length(CUTOFF);

tpt=zeros(nCutOff,1);
npt=tpt; tpn=tpt; npn=tpt; tpMixed=tpt; npMixed=tpt;

for z=1:nCutOff
    currentCutOff=CUTOFF(z);

    fprintf('\n========> Number of trails=%d\n',NTEST);
    fprintf('percentage of samples used in generation of eigensample is =%3.3f%%\n',...
        (currentCutOff)*100);
    fprintf('Number of eigenmode=%d\n\n',NMODE);

    %%%%%%%%%%%%%  DO THE PROJECTION ON TUMOR  %%%%%%%%%%%%%

    tumorSetSize   = length(trueTumorIndex);
    normalSetSize  = length(trueNormalIndex);

    workingPopulationTumorFirst = xraw(:,[trueTumorIndex trueNormalIndex]);

    sizeOfExcludedSet = floor((1-currentCutOff)*length(trueTumorIndex));
    projMatOnTumorPCA    = zeros( tumorSetSize+normalSetSize, NMODE, NTEST);
    projMatOnTumorICA    = zeros( tumorSetSize+normalSetSize, NMODE, NTEST);

    [projMatOnTumorPCA,projMatOnTumorICA]= obtainProjectionMatrix(...
        workingPopulationTumorFirst,...
        tumorSetSize, projMatOnTumorPCA, projMatOnTumorICA, ...
        sizeOfExcludedSet,MULTI_MODE_ADD);

    %%%%%%%%%%  DO THE PROJECTION ON NORMAL %%%%%%%%%%%%%%%%%%%

    %Call it now with flipping tumor/normal around
    workingPopulationNormalFirst = xraw(:,[trueNormalIndex trueTumorIndex]);
    sizeOfExcludedSet = floor((1-currentCutOff)*length(trueNormalIndex));

    projMatOnNormalPCA = zeros(tumorSetSize+normalSetSize, NMODE, NTEST);
    projMatOnNormalICA = zeros(tumorSetSize+normalSetSize, NMODE, NTEST);

    [projMatOnNormalPCA,projMatOnNormalICA] = obtainProjectionMatrix(...
        workingPopulationNormalFirst ,...
        length(trueNormalIndex),...  %notice, fliped
        projMatOnNormalPCA,projMatOnNormalICA, sizeOfExcludedSet,...
        MULTI_MODE_ADD);

    %flip the result of the above call so that tumor sample is first as before
    BPCA=projMatOnNormalPCA([normalSetSize+1:end,1:normalSetSize],:,:);
    BICA=projMatOnNormalICA([normalSetSize+1:end,1:normalSetSize],:,:);

    %%%%%%%%%%% Now that we have projection on normal and on tumor %%%%%
    %%%%%%%%%%%    DO THE ACCURACY ANALYSIS   %%%%%%%%%%%%%%%%%%%%%%%%%%

    fprintf('******  PCA accuracy **************\n');
    [tpt(z),npt(z),tpn(z),npn(z),tpMixed(z),npMixed(z)]=...
        accuracyStats(projMatOnTumorPCA,BPCA,tumorSetSize);

    fprintf('\n******  ICA accuracy **************\n');    
    [tpt(z),npt(z),tpn(z),npn(z),tpMixed(z),npMixed(z)]=...
        accuracyStats(projMatOnTumorICA,BICA,tumorSetSize);

end

if DO_CUT_OF_TEST
    figure;
    plot(100*CUTOFF,tpt,'r')
    hold on;
    plot(100*CUTOFF,tpn,'k')
    plot(100*CUTOFF,tpMixed,'b')
    legend('projection against tumor','projection against normal','projection against combination');
    title(sprintf('Liver data: accuracy of detection of cancer a function of the size\nof the random set used to generate dominant component'));
    xlabel(sprintf('%% of the population used to generate the dominant component'));
    ylabel('accuracy of detecting that a sample is cancerous');
    ylim([-1 103]);
    print -dtiff bladder_detect_cancer_all_algorithms

    figure;
    plot(100*CUTOFF,npt,'r')
    hold on;
    plot(100*CUTOFF,npn,'k'); hold on;
    plot(100*CUTOFF,npMixed,'b');
    legend('projection against tumor','projection against normal','projection against combination');
    title(sprintf('Liver data: accuracy of detection of cancer-free as a function of the size\n of the random set used to generate dominant component'));
    xlabel(sprintf('%% of the population used to generate the dominant component'));
    ylabel('accuracy of detecting that a sample is cancer-free');
    ylim([-1 103]);
    print -dtiff bladder_detect_cancer_free_all_algorithms
    %set(gca,'XTickLabel',{'5';'10';'15';'20';'25';'30';'35';'40';'45';'90';'100'})
end

if 1
    % plot above projections next to each others
    figure;
    %subplot(1,2,1);
    projMat=squeeze(BPCA(:,1,:));
    [nr,nc]=size(projMat);
    pltProjMat=projMat;
    pltProjMat(nr+1,:)=pltProjMat(nr,:);
    pltProjMat(:,nc+1)=pltProjMat(:,nc);
    pcolor(pltProjMat)
    colorbar
    xlabel('Case Number')
    set(gca,'XTick',10:10:NTEST);
    ylabel('Sample number')
    title('Projections on normal Component using PCA')

    %subplot(1,2,2);
    figure;
    projMat=squeeze(projMatOnTumorPCA(:,1,:));
    [nr,nc]=size(projMat);
    pltProjMat=projMat;
    pltProjMat(nr+1,:)=pltProjMat(nr,:);
    pltProjMat(:,nc+1)=pltProjMat(:,nc);
    pcolor(pltProjMat)
    colorbar
    xlabel('Case Number')
    set(gca,'XTick',10:10:NTEST);
    ylabel('Sample number')
    title('Projections on tumor Component PCA')
    
    % plot above projections next to each others
    figure;
    %subplot(1,2,1);
    projMat=squeeze(BICA(:,1,:));
    [nr,nc]=size(projMat);
    pltProjMat=projMat;
    pltProjMat(nr+1,:)=pltProjMat(nr,:);
    pltProjMat(:,nc+1)=pltProjMat(:,nc);
    pcolor(pltProjMat)
    colorbar
    xlabel('Case Number')
    set(gca,'XTick',10:10:NTEST);
    ylabel('Sample number')
    title('Projections on normal Component using ICA')

    %subplot(1,2,2);
    figure;
    projMat=squeeze(projMatOnTumorICA(:,1,:));
    [nr,nc]=size(projMat);
    pltProjMat=projMat;
    pltProjMat(nr+1,:)=pltProjMat(nr,:);
    pltProjMat(:,nc+1)=pltProjMat(:,nc);
    pcolor(pltProjMat)
    colorbar
    xlabel('Case Number')
    set(gca,'XTick',10:10:NTEST);
    ylabel('Sample number')
    title('Projections on tumor Component ICA')
    
end

end

%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% tpt= tumor projection on tumor
% npt= normal projection on tumor
% tpn= tumor projection on normal
% npn= normal projection on normal
% tpMixed= tumor projection on combination
% npMixed= normal projection on combination
%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tpt,npt,tpn,npn,tpMixed,npMixed]=accuracyStats(...
    projMatOnTumor,projMatOnNormal,tumorSetSize)

maxNumberOfModes=size(projMatOnTumor,2);

for i=1:maxNumberOfModes
    fprintf('\n***** results for number of dominant modes of %d\n',i);
    [tpt,npt]=checkProjectionOnTumor(projMatOnTumor,...
        tumorSetSize,...
        i);
    fprintf('result of classification of sample based on correlation with Tumor mode\n');
    fprintf('Correct tumor detect=%%%3.2f, Correct normal detect=%%%3.2f\n',tpt,npt);

    [tpn,npn]=checkProjectionOnNormal(projMatOnNormal,tumorSetSize,i);
    fprintf('\nresult of classification of sample based on correlation with normal mode\n');
    fprintf('Correct tumor detect=%%%3.2f, Correct normal detect=%%%3.2f\n',tpn,npn);

    [tpMixed,npMixed,nUndecided]=checkProjectionOnCombination(projMatOnTumor,...
        projMatOnNormal,tumorSetSize,i);
    fprintf('\nresult of classification of sample based on correlation with combination of modes\n');
    fprintf('Correct tumor detect=%%%3.2f, Correct normal detect=%%%3.2f, Unable to classify=%%%3.2f\n',...
        tpMixed,npMixed,nUndecided);
end

end
%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tumorSamplesDetectOkCount,normalSamplesDetectOkCount]=...
    checkProjectionOnTumor(projMatOnTumor,tumorSetSize,nMode)

tumorSamplesDetectOkCount  = 0;
normalSamplesDetectOkCount = 0;
nCases = size(projMatOnTumor,3);
totalSampleSize = size(projMatOnTumor,1);

for i = 1:nCases  %over number of cases
    for j = 1:totalSampleSize

        pOnTumor = 0;
        for k=1:nMode
            pOnTumor=pOnTumor+projMatOnTumor(j,k,i);
        end

        if pOnTumor>0
            if j<=tumorSetSize
                tumorSamplesDetectOkCount  =  1+ tumorSamplesDetectOkCount;
            end
        else %assume normal, check if correct
            if j>tumorSetSize
                normalSamplesDetectOkCount = 1+ normalSamplesDetectOkCount;
            end
        end
    end
end

%convert to percentages
tumorSamplesDetectOkCount=...
    tumorSamplesDetectOkCount*100/(nCases*tumorSetSize);
normalSamplesDetectOkCount=...
    normalSamplesDetectOkCount*100/(nCases*(totalSampleSize-tumorSetSize));

end
%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tumorSamplesDetectOkCount,normalSamplesDetectOkCount]=...
    checkProjectionOnNormal(projMatOnNormal,tumorSetSize,nMode)

tumorSamplesDetectOkCount   = 0;
normalSamplesDetectOkCount  = 0;

nCases = size(projMatOnNormal,3);
totalSampleSize = size(projMatOnNormal,1);

for i = 1:nCases  %over number of cases
    for j = 1:totalSampleSize

        pOnNormal = 0;
        for k = 1:nMode
            pOnNormal = pOnNormal+projMatOnNormal(j,k,i);
        end

        if pOnNormal>0
            if j>tumorSetSize
                normalSamplesDetectOkCount = 1+ normalSamplesDetectOkCount;
            end
        else %assume tumor, check if correct
            if j<=tumorSetSize
                tumorSamplesDetectOkCount =  1+ tumorSamplesDetectOkCount;
            end
        end
    end
end

%convert to percentages
tumorSamplesDetectOkCount  =...
    tumorSamplesDetectOkCount*100/(nCases*tumorSetSize);
normalSamplesDetectOkCount =...
    normalSamplesDetectOkCount*100/(nCases*(totalSampleSize-tumorSetSize));

end
%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tumorSamplesDetectOkCount,normalSamplesDetectOkCount,...
    notDetectedCount]= checkProjectionOnCombination(projMatOnTumor,...
    projMatOnNormal,...
    tumorSetSize,...
    nMode)

nCases = size(projMatOnTumor,3);
totalSampleSize = size(projMatOnTumor,1);
normalSetSize   = totalSampleSize-tumorSetSize;

tumorSamplesDetectOkCount  = 0;
normalSamplesDetectOkCount = 0;
tumorNotSure  = 0;
normalNotSure = 0;

for i = 1:nCases  %over number of cases
    result=zeros(totalSampleSize,1);
    for j = 1:totalSampleSize

        pOnTumor  = 0;
        for k = 1:nMode
            pOnTumor = pOnTumor+projMatOnTumor(j,k,i);
        end

        pOnNormal = 0;
        for k=1:nMode
            pOnNormal = pOnNormal+projMatOnNormal(j,k,i);
        end

        if  pOnTumor>0 || (pOnTumor-pOnNormal)>0
            result(j) = 1; % tumor
        elseif pOnNormal>0
            result(j) = 0; % normal
        else
            result(j) = -1; %unsure
        end
    end

    tumorSamplesDetectOkCount  = ...
        tumorSamplesDetectOkCount + sum(result(1:tumorSetSize)==1);
    normalSamplesDetectOkCount =...
        normalSamplesDetectOkCount+sum(result(tumorSetSize+1:end)==0);
    tumorNotSure  = tumorNotSure  + sum( result(1:tumorSetSize) == -1);
    normalNotSure = normalNotSure + sum( result(tumorSetSize+1:end) == -1);
end

%convert to percentages

tumorSamplesDetectOkCount  =...
    tumorSamplesDetectOkCount*100/(nCases*tumorSetSize-tumorNotSure);
normalSamplesDetectOkCount =...
    normalSamplesDetectOkCount*100/(nCases*normalSetSize-normalNotSure);
notDetectedCount =...
    (tumorNotSure+normalNotSure)*100/(nCases*totalSampleSize);

end
%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotProjection(B,nFig)

figure(nFig);
projMat=squeeze(B(:,1,:));
[nr,nc]=size(projMat);
pltProjMat=projMat;
pltProjMat(nr+1,:)=pltProjMat(nr,:);
pltProjMat(:,nc+1)=pltProjMat(:,nc);
pcolor(pltProjMat)
colorbar
xlabel('Case Number')
set(gca,'XTick',10:10:100);
ylabel('Sample number')

end

%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%
function [projMatPCA,projMatICA]=obtainProjectionMatrix(xraw,...
    tumorSetSize,...
    projMatPCA,...
    projMatICA,...
    sizeOfExcludedSet,MULTI_MODE_ADD)

if sizeOfExcludedSet>=tumorSetSize
    error('projMat: working set size is too small.');
end

ntests = size(projMatPCA,3);

for k = 1:ntests
    if mod(k,50)==0
        fprintf('case # %d\n',k);
    end

    z = randperm(tumorSetSize);                             %nabbasi
    primaryTumorIndex = z(sizeOfExcludedSet+1:end);

    projMatPCA = getProjectionMatrix(xraw(:,primaryTumorIndex), ...
        xraw, ...
        k, ...
        projMatPCA,0,MULTI_MODE_ADD);

    projMatICA = getProjectionMatrix(xraw(:,primaryTumorIndex), ...
        xraw, ...
        k, ...
        projMatICA,1,MULTI_MODE_ADD);

end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function s=getTitlePrefix(projectionType)

PROJ_ON_TUMOR  = 1;
PROJ_ON_NORMAL = 2;

if projectionType==PROJ_ON_TUMOR
    s='[project on tumor]';
elseif  projectionType==PROJ_ON_NORMAL
    s='[project on normal]';
end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function setLegend(projectionType)

PROJ_ON_TUMOR  = 1;
PROJ_ON_NORMAL = 2;

if projectionType==PROJ_ON_TUMOR
    legend('tumor','normal');
elseif  projectionType==PROJ_ON_NORMAL
    legend('tumor','normal');
end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function projMat=getProjectionMatrix(...
    x,...      %set of tumor samples only to use to generate mode
    xraw,...   %the universe
    k,...      %test case number
    projMat,...   %INOUT, the projection matrix
    isICA,...     % flag. 0 means just do PCA, 1 do ICA
    MULTI_MODE_ADD) %see note in configuration section above


persistent U_first;

nModes=size(projMat,2);

%%%%%%%%%% Generate covariance  THETA matrix %%%%%%%%%%%%%%%%%%%%%
nSamples = size(x,2);

gridSize = size(x,1);
theta    = zeros(nSamples);

for i=1:nSamples
    vi = x(:,i);
    for j=i:nSamples
        vj=x(:,j);
        theta(i,j) = vi'*vj/gridSize/nSamples;
        theta(j,i) = theta(i,j);
    end
end

%%%%%%%%% GENERATE DOMINANT EIGENSAMPLES  PHI VECTORS **********
[u,lam]    = eig(theta);
[lam, ind] = sort(-diag(lam));
%lam        = -lam;
u          = u(:,ind');

for i=1:nSamples
    u(:,i) = u(:,i)/norm(u(:,1),1);   %nabbasi 041407
end

%%%%%%%% Normalized eigenvectors %%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:nModes %nabbasi 041407
    phi(:,i)=zeros(gridSize,1);
    for j=1:nSamples
        phi(:,i)=phi(:,i)+u(j,i)*x(:,j);
    end
    phi(:,i)=phi(:,i)/norm(phi(:,i));
end

% Determine sign for dominant eigenvector...select sign so that the sum
% of the projections of the tumor samples used for the POD is greater
% than 1.

for j=1:nModes   %nabbasi 041407
    sumP1(j)=0.;

    for i=1:nSamples
        sumP1(j)=sumP1(j)+sum(x(:,i)'*phi(:,j));
    end

    %sumP1(j);
    if sumP1(j) < 0
        phi(:,j)=-phi(:,j);
    end
end

% Use ICA to find the independent components
% Transpose the data since ICA uses rows instead of columns for input
if isICA
    fprintf('Performing ICA \n\n')
    U = ICA(phi')';
    for j=1:nModes
        U(:,j)=U(:,j)/norm(U(:,j));
    end

    %make sure the signs of the ICA vectors so that sum of correlation
    %of each ICA vector with all the PCA phi's is positive  nabbasi 061407

    for j=1:nModes

%         theSum=0;
%         for i=1:nModes
%             theSum=theSum+dot(U(:,j),phi(:,i));
%         end
%         if theSum<0
%             %fprintf('Found negative\n');
%             U(:,j) = -U(:,j);
%         end

          %per talk with Dr Lee on June 16,2007. I need to just
          %check if correlation of U(:,j) is positive or not
          %with the first PHI for setting the sign on U as needed.
          
          if  dot(U(:,j),phi(:,1))<0
              U(:,j)=-U(:,j);              
          end
    end

    if k == 1
        U_first = U;
    else
        U = OrderComponents(U, U_first);
    end

    for j=1:nModes
        for i=1:size(xraw,2)
            s=xraw(:,i)'*U(:,j);
            if MULTI_MODE_ADD            
               for z=1:j-1
                   s=s+projMat(i,z,k);
               end
            end
            projMat(i,j,k) = s;
        end
    end
else %no ICA, just do PCA
    for j=1:nModes
        for i=1:size(xraw,2)
            s=xraw(:,i)'*phi(:,j);
            if MULTI_MODE_ADD
               for z=1:j-1
                   s=s+projMat(i,z,k);
               end
            end
            projMat(i,j,k) = s;
        end
    end

end

%Projections...
%for j=1:nModes
%   for i=1:size(xraw,2)
%      projMat(i,j,k) = xraw(:,i)'*U(:,j);
%   end
%end

end
